# --------------------------------------------------------------------------------------
#
#	AppTracker	: Application ToolBar Plugin
#	Purpose		: 	Simple Example of Custom Report
#					
#	
# 	Change Log	:
#   29-01-2020  Martin Loughnane v1.0 - Initial Version
# ----------------------------

# Name of the Script, used as log file name
$global:ScriptName = "Custom Application Toolbar Report"

#region Initialisation


# Automatically set the Debug mode if their are no arguments, AppTracker always passes in Arguments.
$global:debug = ($args.Count -eq 0);
if($debug) { cls }
if($debug) { Write-Host "Running in Debug Mode" }

$global:ScriptStartTime = Get-Date
$global:RunningTimers = @{}

## Set up Module Folder, if it does not exist then exit. 
function Get-ScriptDirectory { $Invocation = (Get-Variable MyInvocation -Scope 1).Value; Split-Path $Invocation.MyCommand.Path; } 
if(-not $debug) { $ModuleFolder = $args[1] } # The physical path to the AppTracker Module folder from Args
else
{
	$scriptdir = Get-ScriptDirectory
	if ((Test-Path $($scriptdir + "\Modules"))) { $ModuleFolder = $scriptdir + "\Modules" }
	else 
	{
		$scriptdir = $scriptdir.Substring(0, $scriptdir.LastIndexOf('\'))
		$scriptdir = $scriptdir.Substring(0, $scriptdir.LastIndexOf('\'))
		if ((Test-Path $($scriptdir + "\Modules"))) { $ModuleFolder = $scriptdir + "\Modules" }
		else { Write-Host "No Module folder found, exiting."; return }
	}
}

if(-not $(Test-Path $ModuleFolder)) {New-Object PSObject -Property @{ AppTrackerMessage="No Module folder found, exiting"; AppTrackerSuccess = $false }; return } 
if(Test-Path $($ModuleFolder + "\AppTracker Helper Functions.ps1")) { .$($ModuleFolder + "\AppTracker Helper Functions.ps1") |Out-Null }

#endregion

#region Global Vars and Args
# Helper Variables
$global:DBNull = [System.DBNull]::Value				# This is the value a Questionnaire will return if it is blank
$global:DateZero = Get-Date "1/1/0001 00:00:00"		# The value a Date will return if it is Null
$global:ATSafeTimeNow = $(Get-Date).ToString("u").Trim("Z")		# A Safe Date Format to save back to AppTracker Questionnaire date fields
$global:TextInfo = [System.Threading.Thread]::CurrentThread.CurrentCulture.TextInfo 	# Text manipulation such as Proper Case


# Every script that is called via AppTracker are passed these four arguments
$ApiUri = 			$args[0]	# The URL to the AppTracker API
$SaveReportFolder = $args[2]	# A physical path that can be used to save report or be used a scratch area
$UserId = 			$args[3]	# The User Id of the user that initiated the script
#endregion

#region **** SETUP HERE *****
#  ---------------------- EDIT HERE  ----------------------

# AppTracker API location for running in debug mode
$debugApiUri = "http://localhost/api/v1/API.svc?wsdl" 

## API Account to run script under
$global:ApiUserName = $(Get-ATApiSecureAcc "DefaultApiAccount" $ModuleFolder).UserName
$global:ApiPassword = $(Get-ATApiSecureAcc "DefaultApiAccount" $ModuleFolder).Password

# Old method to explicty set API User/Pass in file
#$global:ApiUserName = "api_account"; $global:ApiPassword = "Password!"	

# Logging: Use 'Add-Content $Logfile "Some Text" to add info to the log file
$global:LogUsage = $true; 
#  ---------------------- End Edit  ----------------------
#endregion

#region Debug & Logging 
if($debug) 
{
	if ((Test-Path $($scriptdir + "\Logs")) -eq $false) { New-Item $($scriptdir + "\Logs") -ItemType directory | Out-Null }
	$ApiUri = $debugApiUri
	$SaveReportFolder = $scriptdir + "\Logs"
}

$global:Logfile = "{0}\{1} {2}.txt" -f $SaveReportFolder, $ScriptName, $(Get-Date -Format "MMMM yyyy")
if($global:LogUsage)
{
	Add-Content $Logfile $("`r`n{0}`r`nRan At: {1}" -f $ScriptName, $(Get-Date))
	$argPos = 0
	foreach ($arg in $args) { Add-Content $Logfile $("Arg {0}: {1}" -f $argPos, $arg) ; $argPos +=1 };Add-Content $Logfile ""
}
#endregion

#region Register Module & API
try { $ErrorActionPreference = "Stop";  Remove-Module AppTracker-Commands | Out-Null } 
catch { }
finally{ $ErrorActionPreference = "Continue"; }

try {	
	Import-Module $($ModuleFolder + "\AppTracker-Commands.psm1")
	$api = Connect-ATAPI -ApiUri $ApiUri -Username $ApiUserName -Password $ApiPassword -EnableCompression
} catch [system.exception] {
	Add-Content $Logfile $_.Exception.Message
	New-Object PSObject -Property @{ ErrorMessage = $_.Exception.Message }
	New-Object PSObject -Property @{ AppTrackerMessage=$_.Exception.Message; AppTrackerSuccess = $false }
	exit
}
#endregion

#-----------------------------------------------------------------------------------------------------------------
#-------------------	Start of Main Script	------------------------------------------------------------------
#-----------------------------------------------------------------------------------------------------------------

#region Functions
function write-log([System.String]$entry)
{
	$logEntry="[{0:yyyy-MM-dd HH:mm:ss}]`t{1}" -f (get-date),$entry
	if($debug){Write-Host $entry}
	Add-Content $logfile $logentry
}
#endregion 

$ScriptMode	= $args[4]			# Parameter from xml
$IdsSelectedOnGrid= $args[5]	# List of RevisionIds Passed from Grid

 
try 
{
	#Check the script mode paramater is the one we are expecting. If not throw an error.
	
	if($ScriptMode -ne 'Produce_Report')
	{
		throw "Unrecognised Script Mode [$ScriptMode]."
	}

	#Retrieve data from MigrationStudio

	$AppDMCall=@{
		AppTrackerAPI=$api
		RevisionIds=@(0)
		AppFields = @('RevisionId','Process')
		Options=@(
			#To return All Apps
			'ALLAPPS'

			#Use if want all apps including archived
			#'ALLAPPSWITHARCHIVED'
		
			#This will use the internal MS name for fields/questions
			'KEEPINTERNALCOLNAMES'
			
			#To Include a number of users count per App
			'REQUESTS'

			#Can also use
			#'REQUESTSANDDEPENDS'
		)
		QuestionIds=@()
	}

	$appsData=Get-ATApplicationDataMiningReport @AppDMCall

	$appTotals=@{}

	#Do some processing on the retuned data.

	$appsData | ForEach-Object {
		if(!$appTotals.ContainsKey($_.Process))
		{
			$appTotals.Add($_.Process,@{NumberOfApps=0;NumberOfUsers=0})
		}
		$appTotals[$_.Process].NumberOfUsers=$appTotals[$_.Process].NumberOfUsers+$_.'Request Count'
		$appTotals[$_.Process].NumberOfApps=$appTotals[$_.Process].NumberOfApps+1
	}

	#Now wrap the data into a datatable for MigrationStudio to Export to excel.

	[System.Data.DataTable]$myDataTable=new-object System.Data.DataTable

	$null=$myDataTable.Columns.Add('Process')
	$null=$myDataTable.Columns.Add('Number Of Apps')
	$null=$myDataTable.Columns.add('Number Of Users')

	foreach($appTotal in $appTotals.GetEnumerator() | Sort-Object Name)
	{
		$row=$myDataTable.NewRow()
		
		$row['Process']=$appTotal.name
		$row['Number Of Apps']=$appTotal.value.NumberOfApps
		$row['Number Of Users']=$appTotal.value.NumberOfUsers

		$null=$myDataTable.Rows.Add($row)
	}



	#Build and return the object for MS to build the worksheet.

	new-object PSObject -Property @{
		
		#What to call the Excel file
		ExcelFileName='{1:yyyy-MM-dd-HH-mm} {0}.xlsx' -f 'My File',(get-date)
		
		#Where relative to the script to find the template excel file
		ExcelTemplate='.\TemplateExcelFile.xlsx'

		#Our previosuly built Data Table
		DataTable=$myDataTable.DefaultView
		
		#Name of the Worksheet to insert data in table into
		WorkSheet='Data Sheet 1'

		#Starting at row
		RowIndex=1

		#Starting at columns
		ColumnIndex=1

		#Include the data table column names?
		IncludeHeader=$true
	}

	#Establish the calling users email details

	try
	{
		#Get the user object via UserId
		$USerMSAccount=Get-ATUserAccount -AppTrackerAPI $Api -UserId $UserId
		
		#Store email address
		$email=$($USerMSAccount.Email)
	}
	catch
	{
		#default if no email found
		$email='info@apptracker.co.uk'
	}

	#Build and return the object that tells MS to email the previous built worksheet to the calling user.
	
	New-Object PSObject -Property @{

		#Email Subject
		MessageSubject = 'Example Report'
		
		#Email Message (can include Email Template Keywords)
		MessageBody = "Hi [%UserSmartName%] `n[%UserEmail%] `n Your report is ready, `n see attached."
		
		#The email address
		EmailAddresses=$email
	}

	#Build and return the success Message to the MS UI
	
	New-Object PSObject -Property @{
		AppTrackerMessage = 'Report generated and emailed.'
		AppTrackerSuccess = $true 
	}
} 
catch [system.exception] {
	#Build error and write to Log
	$msg="$ScriptName Error reported:`r`n$($_.Exception.Message)`r`n$($_.ScriptStackTrace)`r`n"
	write-log $msg
	#Return Error Message to MS UI
	New-Object PSObject -Property @{
		AppTrackerMessage = $msg 
		AppTrackerSuccess = $false 
	}
}

#-------------------	End of Main Script	------------------------------------------------------------------
$global:ScriptEndTime = Get-Date
##New-Object PSObject -Property @{ InfoMessage = $("Report Complete.<br/><br/><pre>{0}</pre>" -f $(Display-LogTimers)) }
if($debug) { Display-LogTimers -LogToDisk }
else { Display-LogTimers -LogToDisk | Out-Null }

